#include "Window.h"

#include "IPlatformWindow.h"
#include "PaintEvent.h"
#include "MouseEvent.h"
#include "VisualLayer.h"
#include "KeyboardEvent.h"
#include "Image.h"
#include "Screen.h"

namespace mt
{

Window::Window()
    :mWidth(800),
     mHeight(600),
     mBackingImage(std::make_unique<Image<uint8_t> >(mWidth, mHeight)),
     mWidget(Widget::Create())
{
    mWidget->setBounds(mWidth, mHeight);
}

Window::~Window()
{

}

std::unique_ptr<Window> Window::Create()
{
    return std::make_unique<Window>();
}

std::vector<VisualLayer*> Window::GetLayers()
{
    return mWidget->getLayers();
}

void Window::clearPlatformWindow()
{
    mPlatformWindow.reset();
}


void Window::OnMouseEvent(const MouseEvent* event)
{
    mWidget->onMouseEvent(event);
}

void Window::OnKeyboardEvent(const KeyboardEvent* event)
{
    mWidget->onKeyboardEvent(event);
}

void Window::OnPaint(const PaintEvent* event)
{
    mWidget->onPaintEvent(event);
}

void Window::AddWidget(WidgetUPtr widget)
{
    if (mWidget)
    {
        mWidget.reset();
    }
    mWidget = std::move(widget);
    mWidget->setBounds(mWidth, mHeight);
}

Widget* Window::GetWidget() const
{
    return mWidget.get();
}

unsigned Window::GetWidth() const
{
    return mWidth;
}

unsigned Window::GetHeight() const
{
    return mHeight;
}

void Window::SetSize(unsigned width, unsigned height)
{
    mWidth = width;
    mHeight = height;
    mWidget->setBounds(mWidth, mHeight);
}

IPlatformWindow* Window::GetPlatformWindow() const
{
    if (mPlatformWindow)
    {
        return mPlatformWindow.get();
    }
    return nullptr;
}

void Window::SetPlatformWindow(IPlatformWindowPtr window)
{
    mPlatformWindow = std::move(window);
}

Image<uint8_t>* Window::getBackingImage() const
{
    return mBackingImage.get();
}

void Window::map()
{
    if (mPlatformWindow)
    {
        mPlatformWindow->map();
    }
}

void Window::show()
{
    if (mPlatformWindow)
    {
        mPlatformWindow->show();
    }
}

void Window::doPaint(mt::Screen* screen)
{
    if (mPlatformWindow)
    {
        mPlatformWindow->paint(screen);
    }
}

void Window::clear()
{
    if (mPlatformWindow)
    {
        mPlatformWindow->clear();
    }
}

}
